import snappi


def test_types_and_formats():
    config = snappi.Api().config()
    f = config.flows.flow()[-1]
    eth = f.packet.ethernet()[-1]

    # mac validation
    eth.src.value = "11:22:33:aa:bb:cc:22"
    try:
        eth.src.serialize(eth.DICT)
        assert False
    except TypeError:
        assert True

    # ip validation
    ip = f.packet.ipv4()[-1]
    ip.src.value = '1111'
    try:
        ip.src.serialize(ip.DICT)
        assert False
    except TypeError:
        assert True
    ip6 = f.packet.ipv6()[-1]
    ip6.src.value = ":a:1"
    try:
        ip6.src.serialize(ip6.DICT)
        assert False
    except TypeError:
        assert True
 
    #float validation
    f.duration.fixed_seconds.seconds = '10'
    f.duration.fixed_seconds.delay.microseconds = '20'
    try:
        f.duration.fixed_seconds.delay.serialize(f.DICT)
        assert False
    except TypeError:
        assert True
    
    # string validation
    config = snappi.Api().config()
    f = config.flows.flow()[-1]
    f.name = 100
    f.tx_rx.port.tx_name = 'test_tx'
    try:
        f.serialize(f.DICT)
        assert False
    except TypeError:
        assert True

    # integer validation
    d = config.devices.device()[-1]
    eth = d.ethernets.ethernet()[-1]
    eth.connection.port_name = "p1"
    eth.name = 'eth'
    eth.mac = '00:00:00:00:00:00'
    eth.mtu = '1500'
    try:
        eth.serialize(eth.DICT)
        assert False
    except TypeError:
        assert True
    
    eth = f.packet.ethernet()[-1]
    # mac validation with list
    eth.src.values = ['a:b', ':0:0:0:0:0:0:1:']
    try:
        eth.src.serialize(eth.DICT)
        assert False
    except TypeError:
        assert True

    # reset existing choice
    f.size.fixed = 74
    assert f.size.serialize(f.DICT) == {'choice': 'fixed', 'fixed': 74}
    f.size.increment
    assert f.size.serialize(f.DICT) == {
        'choice': 'increment',
        'increment': {'start': 64, 'end': 1518, 'step': 1}
    }
    



import pytest
import snappi


def test_snappi_lists(api):
    """Validate SnappiList object indexing and unpacking
    """
    config = api.config()

    flows = config.flows.flow(name='1')
    assert(flows.__class__ == snappi.FlowIter)
    flow = flows[0]
    flow.tx_rx.port.tx_name = 'p1'
    assert(flow.__class__ == snappi.Flow)
    eth, vlan, vlan1 = flow.packet.ethernet().vlan().vlan()
    vlan.id.value = 1

    flow = config.flows.flow(name='2')[-1]
    flow.tx_rx.port.tx_name = 'p1'
    assert(flow.__class__ == snappi.Flow)
    eth, vlan, ipv4, tcp = flow.packet.ethernet().vlan().ipv4().tcp()
    assert(eth.__class__ == snappi.FlowEthernet)
    assert(vlan.__class__ == snappi.FlowVlan)
    assert(ipv4.__class__ == snappi.FlowIpv4)
    assert(tcp.__class__ == snappi.FlowTcp)
    vlan.id.value = 2

    flow = config.flows.flow(name='3')[-1]
    flow.tx_rx.port.tx_name = 'p1'
    pkt = flow.packet.ethernet().vlan()
    assert(pkt.__class__ == snappi.FlowHeaderIter)
    vlan = pkt[-1]
    vlan.id.value = 3

    flow = config.flows.flow(name='4')[-1]
    flow.tx_rx.port.tx_name = 'p1'
    vlan = flow.packet.ethernet().vlan()[-1]
    assert(vlan.__class__ == snappi.FlowVlan)
    vlan.id.value = 4

    print(config)
    api.set_config(config)
    
    assert (len(config.flows) == 4)
    assert (config.flows[0].name == '1')
    assert (config.flows[1].name == '2')
    assert (config.flows[2].name == '3')
    assert (config.flows[3].name == '4')
    assert (len(config.flows[0].packet) == 3)
    assert (len(config.flows[1].packet) == 4)
    assert (len(config.flows[2].packet) == 2)
    assert (len(config.flows[3].packet) == 2)
    assert (config.flows[0].packet[1].id.value == 1)
    assert (config.flows[1].packet[1].id.value == 2)
    assert (config.flows[2].packet[1].id.value == 3)
    assert (config.flows[3].packet[1].id.value == 4)


if __name__ == '__main__':
    pytest.main(['-vv', '-s', __file__])
